| Safe Haskell | None |
|---|---|
| Language | Haskell2010 |
ADP.Fusion.Base.Set
Description
The Context for a BitSet is the number of bits we should reserve
for the more right-most symbols, which request a number of reserved
bits.
ADPfusion-0.4.0.2: Efficient, high-level dynamic programming.
| Safe Haskell | None |
|---|---|
| Language | Haskell2010 |
ADP.Fusion.Base.Set
Description
The Context for a BitSet is the number of bits we should reserve
for the more right-most symbols, which request a number of reserved
bits.